/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
    Copyright (C) 2018-2021 OpenCFD Ltd.
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "VJFvPatchField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "unitConversion.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

void Foam::VJFvPatchField::calcJump()
{
	if (this->cyclicPatch().owner())
    {
    vectorField aeroCoeff = pvj_bt1;
	vectorField Ujump(patch().Sf()*0.0);
    #include "calculatePVJinitializeForces.H"

    // Initializing output
    vector pvj_forceOut(vector::zero);

    forAll(allN, iN)
    {
       #include "calculatePVJforces.H"
	   // Setting jump
       aeroForce[iN] = -q*ca;
       Ujump[iN] = -q*ca*t1v/(U[iN] & nv)*jumpSign[iN];

	   pvj_forceOut += q*ca*patch().magSf()[iN]*t1v;
    }
	this->setJump(Ujump);

	// Following just used to calculate forces to output
	aeroCoeff = pvj_bn;
    forAll(allN, iN)
    {
       #include "calculatePVJforces.H"
	   // Setting jump
	   pvj_forceOut += q*ca*patch().magSf()[iN]*nv;
    }

	reduce(pvj_forceOut,sumOp<vector>());
	#include <iostream>
    if (this->db().time().value()!=pvj_time && Pstream::myProcNo()==0)
    {
        std::ofstream pvj_stream;
        pvj_stream.open(pvj_outDir+"/" +patch().name(), std::ios_base::app);
		pvj_stream.precision(9);
		pvj_stream.fill('0' );
		pvj_stream.flags(std::ios::fixed);
        pvj_stream << this->db().time().value() << "\t\t" << pvj_forceOut[0] << "\t\t" << pvj_forceOut[1] << "\t\t" << pvj_forceOut[2] << "\n";
		pvj_stream.close();
		pvj_time = this->db().time().value();
    }

    }
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::VJFvPatchField::VJFvPatchField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedJumpFvPatchField<vector>(p, iF),
    pvj_dictName("PVJDict"),
    pvj_point(vector(0.0,0.0,0.0)),
    pvj_t1d(vector(0.0,0.0,0.0))
{}


Foam::VJFvPatchField::VJFvPatchField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedJumpFvPatchField<vector>(p, iF, dict),
    pvj_dictName(dict.getOrDefault<word>("PVJDict", "PVJDict")),
    pvj_point(dict.get<vector>("point")),
    pvj_t1d(dict.get<vector>("t1d"))
{
    const fvMesh& mesh=p.boundaryMesh().mesh();
 	IOdictionary PVJDict
 	    (
 	    		IOobject
 				(
 				pvj_dictName,
 				mesh.time().system(),
 				mesh,
 				IOobject::READ_IF_PRESENT,
 				IOobject::NO_WRITE
 				)
     );

    #include "readPVJ.H"
    if (this->cyclicPatch().owner())
    {
    string command = "mkdir ./postProcessing";
    system(command.c_str());
    command = "mkdir ./postProcessing/PVJ/";
    system(command.c_str());
    command = "mkdir ./postProcessing/PVJ/" + this->db().time().timeName();
    system(command.c_str());
    pvj_outDir = "./postProcessing/PVJ/" + this->db().time().timeName();

    std::ofstream pvj_stream;
    pvj_stream.open(pvj_outDir+"/" +patch().name(), std::ios_base::app);
    pvj_stream << "Time" << "\t\t\t" << "Fx" << "\t\t\t\t" << "Fy" << "\t\t\t\t" << "Fz" << "\n";
    pvj_stream.close();
    }

}


Foam::VJFvPatchField::VJFvPatchField
(
    const VJFvPatchField& rhs,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedJumpFvPatchField<vector>(rhs, p, iF, mapper),
    pvj_dictName(rhs.pvj_dictName),
    pvj_point(rhs.pvj_point),
    pvj_t1d(rhs.pvj_t1d)
{}


Foam::VJFvPatchField::VJFvPatchField
(
    const VJFvPatchField& rhs
)
:
    fixedJumpFvPatchField<vector>(rhs),
    pvj_dictName(rhs.pvj_dictName),
    pvj_point(rhs.pvj_point),
    pvj_t1d(rhs.pvj_t1d)
{}


Foam::VJFvPatchField::VJFvPatchField
(
    const VJFvPatchField& rhs,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedJumpFvPatchField<vector>(rhs, iF),
    pvj_dictName(rhs.pvj_dictName),
    pvj_point(rhs.pvj_point),
    pvj_t1d(rhs.pvj_t1d)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::VJFvPatchField::updateCoeffs()
{
/*    if (this->updated())
    {
        return;
    }
*/
    calcJump();
}


void Foam::VJFvPatchField::write(Ostream& os) const
{
    fixedJumpFvPatchField<vector>::write(os);
    os.writeEntryIfDifferent<word>("PVJDict", "PVJDict", pvj_dictName);
    os.writeEntry("point", pvj_point);
    os.writeEntry("t1d", pvj_t1d);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
   makePatchTypeField
   (
       fvPatchVectorField,
       VJFvPatchField
   );
}

// ************************************************************************* //
