#ifndef VJFvPatchField_H
#define VJFvPatchField_H

#include "fixedJumpFvPatchField.H"
#include "Function1.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                Class VJFvPatchField Declaration
\*---------------------------------------------------------------------------*/

class VJFvPatchField
:
     public fixedJumpFvPatchField<vector>
{
    // Private data
    #include "initializePVJ.H"
    // Private Member Functions

        //- Calculate the fan pressure jump
        void calcJump();


public:

    //- Runtime type information
    TypeName("VJ");


    // Constructors

        //- Construct from patch and internal field
        VJFvPatchField
        (
            const fvPatch&,
            const DimensionedField<vector, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        VJFvPatchField
        (
            const fvPatch&,
            const DimensionedField<vector, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given VJFvPatchField
        //- onto a new patch
        VJFvPatchField
        (
            const VJFvPatchField&,
            const fvPatch&,
            const DimensionedField<vector, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        VJFvPatchField
        (
            const VJFvPatchField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchField<vector>> clone() const
        {
            return tmp<fvPatchField<vector>>
            (
                new VJFvPatchField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        VJFvPatchField
        (
            const VJFvPatchField&,
            const DimensionedField<vector, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchField<vector>> clone
        (
            const DimensionedField<vector, volMesh>& iF
        ) const
        {
            return tmp<fvPatchField<vector>>
            (
                new VJFvPatchField(*this, iF)
            );
        }


    // Member functions

        //- Update the coefficients associated with the patch field
        virtual void updateCoeffs();

        //- Write
        virtual void write(Ostream& os) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

#endif

// ************************************************************************* //
