import numpy
import os
import PVJLib

# Files
outDictionaryName = 'PVJDict'

# Data used for calibration and/or comparison [optional, might be none] 
#inputDataFile = None
inputDataFile = 'Data' + os.sep + 'lamellae_theta_45.txt'
U = 10.0
area = 0.6

# Permeable surface model
pSurfaceDict = dict()

### From data file
#pSurfaceDict['approach'] = 'fromData'
#pSurfaceDict['order'] = 3
#pSurfaceDict['data'] = PVJLib.getData(inputDataFile,U,area)
#pSurfaceDict['ref'] = 'locRef'

### Fully deflective
pSurfaceDict['approach'] = 'fullyDeflective'    # 'fromPorosity', 'fullyDeflective', 'fromData'
pSurfaceDict['theta'] = 45.0

### From porosity
#pSurfaceDict['approach'] = 'fromPorosity'
#pSurfaceDict['beta'] = 0.50
#pSurfaceDict['ref'] = 'velRef'

### ------------------------------------------------------- ### 
#                END OF USER EDITABLE PART                    #
### ------------------------------------------------------- ### 

# Characterising permeable surface
pSurfaceData = PVJLib.getpSurfaceCoefficients(pSurfaceDict)

# Writing OF dictionary
PVJLib.writeOFDict(outDictionaryName,pSurfaceData)

# Showing comparison
from matplotlib import pyplot

if not(inputDataFile==None):
    dataCompare = PVJLib.getData(inputDataFile,U,area)
    pyplot.scatter(dataCompare[:,0],dataCompare[:,1],s=80,c='r',edgecolors='k')
    pyplot.scatter(dataCompare[:,0],dataCompare[:,2],s=80,c='b',edgecolors='k')

alphas = numpy.linspace(0,360,360)
nxc = numpy.power(abs(numpy.cos(alphas*numpy.pi/180.0)),pSurfaceData['gamma'])
cn = PVJLib.getAeroCoeffs(alphas,pSurfaceData['bn'])*nxc
ct1 = PVJLib.getAeroCoeffs(alphas,pSurfaceData['bt1'])*nxc
pyplot.plot(alphas,cn,color='r',linewidth=2)
pyplot.plot(alphas,ct1,color='b',linewidth=2)
pyplot.grid()
pyplot.xlabel(r'$\alpha [^\circ]$',fontsize=15)
pyplot.ylabel(r'$C_i [-]$',fontsize=15)
pyplot.legend([r'$c_n$ PVJ',r'$c_{t1} PVJ$',r'$c_n$ CFD',r'$c_{t1}$ CFD'],fontsize=15)